// Generated by CoffeeScript 1.10.0
(function() {
  var KicadGenerator, QedaElement, QedaLibrary, fs, mkdirp, path, request;

  fs = require('fs');

  mkdirp = require('mkdirp');

  path = require('path');

  request = require('sync-request');

  KicadGenerator = require('./kicad-generator');

  QedaElement = require('./qeda-element');

  QedaLibrary = (function() {
    function QedaLibrary(settings) {
      if (settings == null) {
        settings = {};
      }
      this.elementStyle = 'default';
      this.symbolStyle = 'default';
      this.patternStyle = 'default';
      this.symbol = {
        units: 'mm',
        gridSize: 2.5,
        fontSize: {
          "default": 1,
          refDes: 1,
          name: 1,
          pinNumber: 1,
          pinName: 1
        },
        lineWidth: {
          "default": 0.02
        },
        space: {
          pinName: 1
        }
      };
      this.pattern = {
        densityLevel: 'N',
        decimals: 3,
        tolerance: {
          "default": 0.1,
          fabrication: 0.05,
          placement: 0.05
        },
        clearance: {
          padToSilk: 0.2
        },
        lineWidth: {
          "default": 0.2,
          silkscreen: 0.2,
          assembly: 0.1
        },
        fontSize: {
          "default": 1,
          refDes: 1,
          value: 1
        }
      };
      this.mergeObjects(this, settings);
      this.outlineDefs = [
        {
          regexp: /JEDEC-(.*)/,
          handler: 'jedec'
        }
      ];
      this.elements = [];
    }

    QedaLibrary.prototype.add = function(element) {
      var def, housing, housings, i, j, len, len1, name, newElement, ref, ref1, res, suffix, suffixes;
      def = this.load(element);
      if (def.abstract) {
        console.error("'" + element + "': Cannot add abstract component, use it only as base for others");
        process.exit(1);
      }
      res = [];
      housings = Array.isArray(def.housing) ? def.housing : [def.housing];
      name = def.name;
      for (i = 0, len = housings.length; i < len; i++) {
        housing = housings[i];
        suffixes = '';
        if (typeof housing === 'string') {
          def.housing = def[housing];
          suffixes = ((ref = def[housing]) != null ? ref.suffix : void 0) != null ? def[housing].suffix : housing;
        } else if (housing.suffix != null) {
          suffixes = housing.suffix;
        }
        if (!Array.isArray(suffixes)) {
          suffixes = [suffixes];
        }
        def.name = name + suffixes[0];
        if (suffixes.length > 1) {
          if (def.alias == null) {
            def.alias = [];
          }
          if (!Array.isArray(def.alias)) {
            def.alias = [def.alias];
          }
          ref1 = suffixes.slice(1);
          for (j = 0, len1 = ref1.length; j < len1; j++) {
            suffix = ref1[j];
            def.alias.push(name + suffix);
          }
        }
        newElement = new QedaElement(this, def);
        this.elements.push(newElement);
        res.push(newElement);
      }
      return res;
    };

    QedaLibrary.prototype.calculate = function() {
      var element, i, j, l, len, len1, len2, prop, ref, ref1, ref2, symbol;
      if (this._calculated == null) {
        this._calculated = false;
      }
      if (this._calculated) {
        return;
      }
      ref = this.elements;
      for (i = 0, len = ref.length; i < len; i++) {
        element = ref[i];
        element.render();
      }
      for (prop in this.symbol.fontSize) {
        this.symbol.fontSize[prop] *= this.symbol.gridSize;
      }
      for (prop in this.symbol.lineWidth) {
        this.symbol.lineWidth[prop] *= this.symbol.gridSize;
      }
      for (prop in this.symbol.space) {
        this.symbol.space[prop] *= this.symbol.gridSize;
      }
      ref1 = this.elements;
      for (j = 0, len1 = ref1.length; j < len1; j++) {
        element = ref1[j];
        ref2 = element.symbols;
        for (l = 0, len2 = ref2.length; l < len2; l++) {
          symbol = ref2[l];
          symbol.resize(this.symbol.gridSize);
        }
      }
      return this._calculated = true;
    };

    QedaLibrary.prototype.generateKicad = function(name) {
      var kicad;
      this.calculate();
      kicad = new KicadGenerator(this);
      return kicad.generate(name);
    };

    QedaLibrary.prototype.load = function(element) {
      var baseDef, baseElement, def, elementJson, localFile, res;
      elementJson = element.toLowerCase() + '.json';
      localFile = './library/' + elementJson;
      if (!fs.existsSync(localFile)) {
        res = request('GET', 'https://raw.githubusercontent.com/qeda/library/master/' + elementJson, {
          timeout: 3000
        });
        if (res.statusCode === 200) {
          mkdirp.sync(path.dirname(localFile));
          fs.writeFileSync(localFile, res.body);
          console.log("Loading '" + element + "': OK");
        } else {
          console.error("Loading '" + element + "': Error (" + res.statusCode + ")");
          process.exit(1);
        }
      }
      def = JSON.parse(fs.readFileSync(localFile));
      if (def.base != null) {
        baseElement = def.base;
        delete def.base;
        if (path.dirname(baseElement) === '.') {
          baseElement = path.dirname(element) + '/' + baseElement;
        }
        baseDef = this.load(baseElement);
        if (baseDef.abstract) {
          delete baseDef.abstract;
        }
        this.mergeObjects(baseDef, def);
        def = baseDef;
      }
      return def;
    };

    QedaLibrary.prototype.mergeObjects = function(dest, src) {
      var k, results, v;
      results = [];
      for (k in src) {
        v = src[k];
        if (typeof v === 'object' && (!Array.isArray(v)) && dest.hasOwnProperty(k)) {
          results.push(this.mergeObjects(dest[k], v));
        } else {
          results.push(dest[k] = v);
        }
      }
      return results;
    };

    QedaLibrary.prototype.setElementStyle = function(style) {
      style = style.toLowerCase();
      this.elementStyle = style;
      if (this.symbolStyle === 'default' || style === 'default') {
        this.symbolStyle = style;
      }
      if (this.patternStyle === 'default' || style === 'default') {
        return this.patternStyle = style;
      }
    };

    QedaLibrary.prototype.setPatternStyle = function(style) {
      return this.patternStyle = style.toLowerCase();
    };

    QedaLibrary.prototype.setSymbolStyle = function(style) {
      return this.symbolStyle = style.toLowerCase();
    };

    return QedaLibrary;

  })();

  module.exports = QedaLibrary;

}).call(this);
